/* vim: set noexpandtab fileencoding=utf-8 nomodified nowrap textwidth=270 foldmethod=marker foldmarker={{{,}}} foldcolumn=4 ruler showcmd lcs=tab\:|- list: */
#include "DS1307.h"
// #include "Wire.h"
#define DS1307_I2C_ADDRESS 0x68
#include <Arduino.h>

uint8_t DS1307::decToBcd(uint8_t val) {		// {{{ Convert normal decimal numbers to binary coded decimal
	return( (val/10*16) + (val%10) );
}	// }}}
uint8_t DS1307::bcdToDec(uint8_t val) {		// {{{ Convert binary coded decimal to normal decimal numbers
	return( (val/16*10) + (val%16) );
}	// }}}

void DS1307::decTo2char(uint8_t val, char *c) {		// {{{ Convert normal decimal number 00..99 to two char
	c[0]= '0'+val/10;
	c[1]= '0'+val%10;
}	// }}}
void DS1307::bcdTo2char(uint8_t val, char *c) {		// {{{ Convert binary coded decimal number 00..99 to two char
	c[0]= '0'+((val>>4)&0x0F);
	c[1]= '0'+(val&0x0F);
}	// }}}
DS1307::DS1307() : previous_millis(0),year(0), month(1), day(1), hour(0), minute(0), second(0), dayOfWeek(6), present(false) { 	// {{{
		strcpy(textDate,"20xx.xx.xx");
		strcpy(textTime,"xx:xx:xx");
//		previous_millis=current_millis;
}	// }}}
bool DS1307::isPresent() { 	// {{{ test if HW is present
	Wire.beginTransmission(DS1307_I2C_ADDRESS);
	present = (0 == Wire.endTransmission());
	return present;
}	// }}}
void DS1307::setDateTime( uint8_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t minute, uint8_t second, uint8_t dayOfWeek) {		// {{{ year:00..99 dOW: 0=Sunday, 1=Monday, ..., 7=Sunday
	this->year = year; 
	this->month = month; 
	this->day = day; 
	this->hour = hour; 
	this->minute = minute; 
	this->second = second; 
	this->dayOfWeek = dayOfWeek;
	setDateTime();
}	// }}}
void DS1307::setDateTime(char *Date, char *Time, char dOW) {	// {{{ "2023.09.07" "15:16:00" 4/"4"
	year=bcdToDec((Date[2]-'0')*16+(Date[3]-'0'));
	month=bcdToDec((Date[5]-'0')*16+(Date[6]-'0'));
	day=bcdToDec((Date[8]-'0')*16+(Date[9]-'0'));
	hour=bcdToDec((Time[0]-'0')*16+(Time[1]-'0'));
	minute=bcdToDec((Time[3]-'0')*16+(Time[4]-'0'));
	second=bcdToDec((Time[6]-'0')*16+(Time[7]-'0'));
	dayOfWeek=dOW;
	if (dayOfWeek>='0') dayOfWeek-='0';
	setDateTime();
}	// }}}
void DS1307::setDateTime() {		// {{{ year:00..99 dOW: 0=Sunday, 1=Monday, ..., 7=Sunday
	uint8_t tmp;
	dayOfWeek = dayOfWeek%7;
	if (dayOfWeek == 0) dayOfWeek=7;
	if (year > 99) year=year%100;
	if (isPresent()) {
		Wire.beginTransmission(DS1307_I2C_ADDRESS);
		Wire.write(0); // set next input to start at the seconds register
		tmp=decToBcd(second); // set second 00..59
		bcdTo2char(tmp,&textTime[6]);
		Wire.write(tmp);
		tmp=decToBcd(minute); // set minute 00..59
		bcdTo2char(tmp,&textTime[3]);
		Wire.write(tmp);
		tmp=decToBcd(hour); // set hour 00..23 ... bit 6=0 -> 24 hour time
		bcdTo2char(tmp,&textTime[0]);
		Wire.write(tmp);
		tmp=decToBcd(dayOfWeek); // set day of week 1..7 (1=Monday, 7=Sunday)
		Wire.write(tmp);
		tmp=decToBcd(day); // set date 01..31
		bcdTo2char(tmp,&textDate[8]);
		Wire.write(tmp);
		tmp=decToBcd(month); // set month 01..12 bit 7=century (0)
		bcdTo2char(tmp,&textDate[5]);
		Wire.write(tmp);
		tmp=decToBcd(year); // set year 00..99 (roll over 99->00 flip bit 7 in month)
		bcdTo2char(tmp,&textDate[2]);
		Wire.write(tmp);
		Wire.endTransmission();
	} else {
		updateText();
	};
	previous_millis=current_millis;	// time is up to date, temperature left old
}	// }}}
bool DS1307::readDateTime() {		// {{{ updates variables if possible
	uint8_t tmp;
	if (!isPresent()) return false;
	Wire.beginTransmission(DS1307_I2C_ADDRESS);
	Wire.write(0x00); // set DS1307 register pointer to 00h
	Wire.endTransmission();
	Wire.requestFrom(DS1307_I2C_ADDRESS, 7);
//	Wire.beginTransmission(DS1307_I2C_ADDRESS);
//	Wire.write(0); // set DS1307 register pointer to 00h
//	Wire.endTransmission();
//	Wire.requestFrom(DS1307_I2C_ADDRESS, 7);
	// request seven bytes of data from DS1307 starting from register 00h
	tmp = Wire.read();
	bcdTo2char(tmp,&textTime[6]);
	second = bcdToDec(tmp);
	tmp = Wire.read();
	bcdTo2char(tmp,&textTime[3]);
	minute = bcdToDec(tmp);
	tmp = Wire.read();
	bcdTo2char(tmp,&textTime[0]);
	hour = bcdToDec(tmp);		// 40+ hours = 12 hour time = something is bad anyway
	tmp = Wire.read();
	dayOfWeek = bcdToDec(tmp);	// 1=Monday, 7=Sunday
	tmp = Wire.read();
	bcdTo2char(tmp,&textDate[8]);
	day = bcdToDec(tmp);
	tmp = Wire.read();
	bcdTo2char(tmp,&textDate[5]);
	month = bcdToDec(tmp & 0x7F);	// bit 7=century (0)
	tmp = Wire.read();
	bcdTo2char(tmp,&textDate[2]);
	year = bcdToDec(tmp);
	previous_millis=current_millis;		// time is up to date
	return true;
}	// }}}
uint8_t DS1307::daysOfMonths[]={0,31,28,31,30,31,30,31,31,30,31,30,31};	// no days in 0.month; 28 days in 2.month
void DS1307::update() {		// {{{ do periodic update of time structures
	if ( (previous_millis==0) || (current_millis - previous_millis) >= 1000UL ) {	// update time every second
		if ( ! readDateTime()) {		// try the best
			if ( ++second >= 60) {	// {{{
				second = 0;
				if (++minute >=60) {
					minute = 0;
					if (++hour >=24) {
						hour=0;
						if (++dayOfWeek>7) dayOfWeek=1;
						if (++day > daysOfMonths[month]) {
							day=1;
							if (++month>12) {
								month=1;
								++year;
							};
						};
					};
				};
			};	// }}}
			updateText();
		};
		previous_millis=current_millis;
	};
}	// }}}
void DS1307::updateText() {	// {{{ parse numbers to textTime & textDate
	decTo2char(second,&textTime[6]);
	decTo2char(minute,&textTime[3]);
	decTo2char(hour,&textTime[0]);
	decTo2char(day,&textDate[8]);
	decTo2char(month,&textDate[5]);
	decTo2char(year,&textDate[2]);
}	// }}}

/*	// {{{
	RTC.year=23;
	RTC.month=9;
	RTC.day=7;
	RTC.hour=15;
	RTC.minute=16;
	RTC.second=00;
	RTC.dayOfWeek=4;
	RTC.setDateTime();
*/	// }}}
	
// RTC.setDateTime(23,9,7,    15,16,00,   4);
