/* vim: set noexpandtab fileencoding=utf-8 nomodified nowrap textwidth=270 foldmethod=marker foldmarker={{{,}}} foldcolumn=4 ruler showcmd lcs=tab\:|- list: */
//#define VARIANT "typ-X"
#define STR_HELPER(x) #x
#define STR(x) STR_HELPER(x)

#ifdef VARIANT
	#pragma message "VARIANT='" STR(VARIANT ) "'"
#else
	#error "No VARIANT"
#endif

#define VERZE "Meraduino_I v.0.1 (RTCx+SD/no temp) var." STR(VARIANT) " build at " DATE " " __TIME__
/*
* SD card datalogger

* The circuit:
	* SD card attached to SPI bus as follows:
		* MOSI - pin 11
		* MISO - pin 12
		* CLK - pin 13
		* CS - pin 10 (for MKRZero SD: SDCARD_SS_PIN)
	* RTC at I2C
		* SDA - A4
		* SCK - A5
	* OneWire at pin 2
	* Green LED vs. GND at pin 3
	* Red   LED vs. GND at pin 4
	* Green BTN vs. GND at pin 5
	* Red   BTN vs. GND at pin 6
	* Photoresistor at A0 vs GND (INPUT_PULLUP)
*/

#define Green_LED 3
#define Red_LED 4
#define Green_BTN  5
#define Red_BTN  6
#define Photoresistor A0

#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include "current_millis.h"

const int chipSelect = 10;

#if VARIANT == 'A'
	#include "DS1307.h"
	DS1307 RTCx=DS1307();
#endif
#if VARIANT == 'B'
	#include "PCF8563.h"
	PCF8563 RTCx=PCF8563();
#endif

#include <OneWire.h>
#include <DallasTemperature.h>

#define BUS_PIN 2
#define ADR_SIZE 8 //byte

// D13b80
OneWire Bus(BUS_PIN);
DallasTemperature Sensors(&Bus);
DeviceAddress DevAdr;

uint8_t nSensors = 0;


void printAddress(DeviceAddress devAdr, uint8_t idx)
{
  Serial.print("Adresa senzoru ");
  Serial.print(idx + 1);
  Serial.print(": ");

  for (uint8_t i = 0; i < ADR_SIZE; i++)
  {
    if (devAdr[i] < 16) Serial.print("0");
    Serial.print(devAdr[i], HEX);
  }
  Serial.println();
}
void DS18B20_printAddresses_File() {	// {{{ prints line to file
		if (!SD.begin(chipSelect)) {
			Serial.println(F("Card failed, or not present"));
		} else {	// {{{
			// open the file. note that only one file can be open at a time,
			// so you have to close this one before opening another.
			// Arduino SD knows only 8.3 files ...
//			char fname[]="01234567.9AB";
			char fname[]="MERrrmmX.LOG";
			fname[3]=RTCx.textDate[2];
			fname[4]=RTCx.textDate[3];
			fname[5]=RTCx.textDate[5];
			fname[6]=RTCx.textDate[6];
			fname[7]=VARIANT;
			File dataFile = SD.open(fname, FILE_WRITE);	// open for append
			// if the file is available, write to it:
			if (dataFile) {
				dataFile.print("## ");
				dataFile.print(RTCx.textDate);
				dataFile.print(" ");
				dataFile.print(RTCx.textTime);
				dataFile.print(": ");
				dataFile.print(digitalRead(Green_LED)?"+ ":"- ");
				dataFile.print("## ");
				dataFile.print(nSensors);
				dataFile.println(" senzoru");
				//vypis adres senzoru
				for (uint8_t i = 0; i < nSensors; i++)
					{
					Sensors.getAddress(DevAdr, i);
					printAddress(DevAdr, i);
					dataFile.print("#  ");
					dataFile.print(RTCx.textDate);
					dataFile.print(" ");
					dataFile.print(RTCx.textTime);
					dataFile.print(": ");
					dataFile.print(digitalRead(Green_LED)?"+ ":"- ");
					dataFile.print("#  ");
					if (i+1<10) dataFile.print("0");
					dataFile.print(i + 1);
					dataFile.print(": ");
					for (uint8_t j = 0; j < ADR_SIZE; j++)
						{
						if (DevAdr[j] < 16) dataFile.print("0");
							dataFile.print(DevAdr[j], HEX);
						}
					dataFile.println();
					}
				dataFile.close();
				// print to the serial port too:
				Serial.println(F("Data written"));
			} else {
				Serial.print("error opening ");
				Serial.println(fname);
			};
		};		// }}}
}	// }}}

void DS18B20_print_Serial()
{
  Sensors.requestTemperatures();

  for (uint8_t i = 0; i < nSensors; i++)
  {
//    Serial.print("Teplota cidel ");
//    Serial.print(i + 1);
    Serial.print(" ");
    Serial.print(Sensors.getTempCByIndex(i));
  }
  Serial.println();
}
void DS18B20_print_File() {	// {{{ prints line to file
		if (!SD.begin(chipSelect)) {
			Serial.println(F("Card failed, or not present"));
		} else {	// {{{
			// open the file. note that only one file can be open at a time,
			// so you have to close this one before opening another.
			// Arduino SD knows only 8.3 files ...
//			char fname[]="01234567.9AB";
			char fname[]="MERrrmmX.LOG";
			fname[3]=RTCx.textDate[2];
			fname[4]=RTCx.textDate[3];
			fname[5]=RTCx.textDate[5];
			fname[6]=RTCx.textDate[6];
			fname[7]=VARIANT;
			File dataFile = SD.open(fname, FILE_WRITE);	// open for append
			// if the file is available, write to it:
			if (dataFile) {
				dataFile.print(RTCx.textDate);
				dataFile.print(" ");
				dataFile.print(RTCx.textTime);
				dataFile.print(": ");
				dataFile.print(digitalRead(Green_LED)?"+ ":"- ");
				dataFile.print(1024-analogRead(Photoresistor));
				dataFile.print(" ");
				Sensors.requestTemperatures();
				
				for (uint8_t i = 0; i < nSensors; i++)
					{
					dataFile.print(" ");
					dataFile.print(Sensors.getTempCByIndex(i));
					}
				dataFile.println();
				dataFile.close();
				// print to the serial port too:
				Serial.println(F("Data written"));
			} else {
				Serial.print("error opening ");
				Serial.println(fname);
			};
		};		// }}}
}	// }}}
void setup() {
	pinMode(Green_LED, OUTPUT);
	digitalWrite(Green_LED,LOW);
	pinMode(Red_LED, OUTPUT);
	digitalWrite(Red_LED,LOW);
	pinMode(Green_BTN , INPUT_PULLUP);
	pinMode(Red_BTN , INPUT_PULLUP);
	pinMode(Photoresistor, INPUT_PULLUP);
	
	current_millis=millis();
	Serial.begin(115200);
	Serial.println(F(VERZE));
	Wire.begin();
	Wire.setWireTimeout(3000,true);
/*
	Serial.println("Press a key for start");
	while ( ! Serial.available()) {};
	while (Serial.available()) Serial.read();
	Serial.println("Press a key for setting time");
	while ( ! Serial.available()) {};
	while (Serial.available()) Serial.read();
	RTCx.setDateTime(23,10,3,    9,47,00,   2);
/**/
	Serial.print("Initializing SD card...");
	RTCx.update();
	if (!SD.begin(chipSelect)) {
		Serial.println("Card failed, or not present");
	} else {	// {{{
		// open the file. note that only one file can be open at a time,
		// so you have to close this one before opening another.
		// Arduino SD knows only 8.3 files ...
//		char fname[]="01234567.9AB";
		char fname[]="BOOT_LOG.LOG";
		File dataFile = SD.open(fname, FILE_WRITE);	// open for append
		// if the file is available, write to it:
		if (dataFile) {
			dataFile.print(RTCx.textDate);
			dataFile.print(" ");
			dataFile.print(RTCx.textTime);
			dataFile.print(": ");
			dataFile.print("Booting ");
			dataFile.println(VERZE);
			dataFile.close();
			// print to the serial port too:
		} else {
			Serial.print("error opening ");
			Serial.println(fname);
		};
	};		// }}}
	Serial.println(RTCx.textTime);
	
  Sensors.begin();

  //pocet senzoru
  nSensors = Sensors.getDeviceCount();

  //vypis poctu senzoru
  Serial.print("Pocet senzoru: ");
  Serial.println(nSensors);

  //vypis adres senzoru
  for (uint8_t i = 0; i < nSensors; i++)
  {
    Sensors.getAddress(DevAdr, i);
    printAddress(DevAdr, i);
  }
  Serial.println("****************************");
	if (!SD.begin(chipSelect)) {
		Serial.println(F("Card failed, or not present"));
	} else {	// {{{
		// open the file. note that only one file can be open at a time,
		// so you have to close this one before opening another.
		// Arduino SD knows only 8.3 files ...
//		char fname[]="01234567.9AB";
		char fname[]="MERrrmmX.LOG";
		fname[3]=RTCx.textDate[2];
		fname[4]=RTCx.textDate[3];
		fname[5]=RTCx.textDate[5];
		fname[6]=RTCx.textDate[6];
		fname[7]=VARIANT;
		File dataFile = SD.open(fname, FILE_WRITE);	// open for append
		// if the file is available, write to it:
		if (dataFile) {
			dataFile.print("## ");
			dataFile.print(RTCx.textDate);
			dataFile.print(" ");
			dataFile.print(RTCx.textTime);
			dataFile.print(": ");
			dataFile.print(digitalRead(Green_LED)?"+ ":"- ");
			dataFile.print(1024-analogRead(Photoresistor));
			dataFile.print(" ");
			dataFile.print(" Booting ");
			dataFile.println(VERZE);
			dataFile.close();
			// print to the serial port too:
		} else {
			Serial.print("error opening ");
			Serial.println(fname);
		};
	};		// }}}
	DS18B20_printAddresses_File();
	DS18B20_print_File();
	Serial.println("setup ends");
}	// }}}

unsigned long Green_millis,Red_millis, debounce = 1000UL;
bool Green_block, Red_block;
unsigned long last_millis, interval = 600000UL;
void loop() {	// {{{
	current_millis=millis();
	RTCx.update();
	if (Green_block) { if ( (current_millis - Green_millis) >= debounce) { Green_block = false; }; };
	if (Red_block) { if ( (current_millis - Red_millis) >= debounce) { Red_block = false; }; };
	if (!Green_block && !digitalRead(Green_BTN)) { Green_block=true; Green_millis=current_millis;digitalWrite(Green_LED,!digitalRead(Green_LED));};
	if (!Red_block && !digitalRead(Red_BTN)) {
		Red_block=true; Red_millis=current_millis;
		if (! digitalRead(Red_LED)) {
			DS18B20_print_File();
		} else {
			DS18B20_printAddresses_File();
			DS18B20_print_File();
		};
		digitalWrite(Red_LED,!digitalRead(Red_LED));
	};
	if ( (current_millis - last_millis) >= interval) {
		last_millis = current_millis;
		Serial.print("\r");
		if (RTCx.isPresent()) Serial.print("RTC "); else Serial.print("--- ");
		Serial.print(RTCx.textDate);
		Serial.print(" ");
		Serial.print(RTCx.textTime);
		Serial.print(" ");
		Serial.print(digitalRead(Green_LED)?"+ ":"- ");
		Serial.print(1024-analogRead(Photoresistor));
		Serial.print(" ");
		
		
		DS18B20_print_Serial();
		if (! digitalRead(Red_LED)) {
			DS18B20_print_File();
		};
	};
}	// }}}
